@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -29,18 +29,26 @@ use warnings;
 
 use parent qw(NeedRestart::UI);
 use NeedRestart qw(:ui);
-use Debconf::Client::ConfModule qw(:all);
+use Sort::Naturally;
 
 use constant {
     DCTMPL => '/usr/share/needrestart/needrestart.templates',
 };
 
+BEGIN {
+    die __PACKAGE__." is not supported as normal user!\n" if($<);
+}
+
+use Debconf::Client::ConfModule qw(:all);
+
 version('2.0');
 capb('backup');
 
 needrestart_ui_register(__PACKAGE__, NEEDRESTART_PRIO_HIGH);
 
 sub dcres(@) {
+    return unless(scalar @_);
+
     my ($rc, @bulk) = @_;
 
     if($rc != 0 && $rc != 30) {
@@ -87,8 +95,6 @@ sub announce_abi {
 
 sub announce_ver {
     my $self = shift;
-    my $kversion = shift;
-    my $kmessage = shift;
 
     $self->_announce('needrestart/ui-kernel_announce_ver', @_);
 }
@@ -102,17 +108,32 @@ sub notice {
 }
 
 
-sub query_pkgs($$$$$) {
+sub query_pkgs($$$$$$) {
     my $self = shift;
     my $out = shift;
     my $defno = shift;
     my $pkgs = shift;
+    my $overrides = shift;
     my $cb = shift;
 
     # prepare checklist array
-    my @l = sort keys %$pkgs;
+    my @l = nsort keys %$pkgs;
 
-    dcres(set('needrestart/ui-query_pkgs', join(', ', ($defno ? () : @l) )));
+    # apply rc selection overrides
+    my @selected = ();
+    foreach my $pkg (@l) {
+	my $found;
+	foreach my $re (keys %$overrides) {
+	    next unless($pkg =~ /$re/);
+
+	    push(@selected, $pkg) if($overrides->{$re});
+	    $found++;
+	    last;
+	}
+
+	push(@selected, $pkg) unless($defno || $found);
+    }
+    dcres(set('needrestart/ui-query_pkgs', join(', ', @selected)));
 
     dcres( subst('needrestart/ui-query_pkgs', 'OUT', $out) );
     dcres( subst('needrestart/ui-query_pkgs', 'PKGS', join(', ', @l)) );
@@ -128,12 +149,65 @@ sub query_pkgs($$$$$) {
     # Debconf kills STDOUT... try to restore it
     open(STDOUT, '> /dev/tty') || open(STDOUT, '>&2');
 
+    # user has canceled
+    return unless(defined($s));
+    return if($r eq 'backup');
+
     # get selected rc.d script
     my @s = split(/, /, $s);
 
+    # restart each selected service script
+    &$cb($_) for @s;
+}
+
+sub query_conts($$$$$$) {
+    my $self = shift;
+    my $out = shift;
+    my $defno = shift;
+    my $pkgs = shift;
+    my $overrides = shift;
+    my $cb = shift;
+
+    # prepare checklist array
+    my @l = nsort keys %$pkgs;
+
+    # apply rc selection overrides
+    my @selected = ();
+    foreach my $pkg (@l) {
+	my $found;
+	foreach my $re (keys %$overrides) {
+	    next unless($pkg =~ /$re/);
+
+	    push(@selected, $pkg) if($overrides->{$re});
+	    $found++;
+	    last;
+	}
+
+	push(@selected, $pkg) unless($defno || $found);
+    }
+    dcres(set('needrestart/ui-query_conts', join(', ', @selected)));
+
+    dcres( subst('needrestart/ui-query_conts', 'OUT', $out) );
+    dcres( subst('needrestart/ui-query_conts', 'CONTS', join(', ', @l)) );
+    dcres( fset('needrestart/ui-query_conts', 'seen', 0) );
+    dcres( settitle('needrestart/ui-query_conts_title') );
+    dcres( input('critical', 'needrestart/ui-query_conts') );
+    my ($r) = dcres( go );
+
+    my ($s) = dcres( get('needrestart/ui-query_conts') );
+
+    stop;
+
+    # Debconf kills STDOUT... try to restore it
+    open(STDOUT, '> /dev/tty') || open(STDOUT, '>&2');
+
     # user has canceled
+    return unless(defined($s));
     return if($r eq 'backup');
 
+    # get selected rc.d script
+    my @s = split(/, /, $s);
+
     # restart each selected service script
     &$cb($_) for @s;
 }
